unit Link;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TLinkState = (lsOut, lsOver, lsClicked);
  TLinkStyle = (lsSite, lsEmail);
  TLink = class(TCustomLabel)
  private
    FMouseIn: boolean;
    FClicked: boolean;
    FAddress: string;
    FMainFont, FOverFont, FClickedFont: TFont;
    FOnLink: TNotifyEvent;
    FLinkStyle: TLinkStyle;
    procedure OnFontChange(Sender: TObject);
    procedure SetOverFont(v: TFont);
    procedure SetClickedFont(v: TFont);
    procedure SetMainFont(v: TFont);
    procedure SetAddress(v: string);
  protected
    procedure WMMOUSEMOVE(var Msg: TWMMOUSEMOVE); message WM_MOUSEMOVE;
    procedure CMMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
    procedure WMLBUTTONDOWN(var Msg: TWMLBUTTONDOWN); message WM_LBUTTONDOWN;
    procedure WMLBUTTONUP(var Msg: TWMLBUTTONUP); message WM_LBUTTONUP;
    procedure Update; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ClickLink;
  published
    property Align;
    property Caption;
    property Transparent;
    property Visible;
    property PopupMenu;
    property ParentShowHint;
    property Layout;
    property Alignment;
    property AutoSize;
    property Address: string read FAddress write SetAddress;
    property MainFont: TFont read FMainFont write SetMainFont;
    property OverFont: TFont read FOverFont write SetOverFont;
    property ClickedFont: TFont read FClickedFont write SetClickedFont;
    property LinkStyle: TLinkStyle read FLinkStyle write FLinkStyle;

    property OnLink: TNotifyEvent read FOnLink write FOnLink;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

uses ShellAPI;

procedure Register;
begin
  RegisterComponents('Samples', [TLink]);
end;

procedure TLink.OnFontChange(Sender: TObject);
begin
  Update;
end;
procedure TLink.SetOverFont(v: TFont);
begin
  FOverFont := v;
  Update;
end;
procedure TLink.SetClickedFont(v: TFont);
begin
  FClickedFont := v;
  Update;
end;
procedure TLink.SetMainFont(v: TFont);
begin
  FMainFont := v;
  Update;
end;
procedure TLink.SetAddress(v: string);
begin
  if pos('@', v) > 0
    then FLinkStyle := lsEmail
    else FLinkStyle := lsSite;
  FAddress := v;  
end;
procedure TLink.WMMouseMove(var Msg: TWMMouseMove);
begin
  if (((MK_LBUTTON and Msg.Keys) <> 0) = FClicked)
    and (FMouseIn) then Exit;
  FClicked := (MK_LBUTTON and Msg.Keys) <> 0;
  FMouseIn := true;
  Update;
end;
procedure TLink.CMMouseLeave(var Msg: TMessage);
begin
  FMouseIn := false;
  Update;
end;
procedure TLink.WMLBUTTONDOWN(var Msg: TWMLBUTTONDOWN);
begin
  FClicked := true;
  Update;
end;
procedure TLink.WMLBUTTONUP(var Msg: TWMLBUTTONUP);
begin
  FClicked := false;
  Update;
  ClickLink;
end;
constructor TLink.Create(AOwner: TComponent);
begin
  inherited;

  FMainFont := TFont.Create;
  FMainFont.OnChange := OnFontChange;
  FMainFont.Color := clNavy;
  FMainFont.Style := [fsUnderline];

  FOverFont := TFont.Create;
  FOverFont.OnChange := OnFontChange;
  FOverFont.Color := clBlue;
  FOverFont.Style := [fsUnderline];

  FClickedFont := TFont.Create;
  FClickedFont.OnChange := OnFontChange;
  FClickedFont.Color := clRed;
  Cursor := crHandPoint;
  Caption := 'delphi4all';
  Address := 'http://delphi4all.narod.ru';
  FMouseIn := false;
  FClicked := false;
  Update;
end;
destructor TLink.Destroy;
begin
  inherited;
  FMainFont.Destroy;
  FOverFont.Destroy;
  FClickedFont.Destroy;
end;
procedure TLink.ClickLink;
begin
  if Assigned(FOnLink) then FOnLink(self);
  if FLinkStyle = lsSite
    then ShellExecute(Application.Handle, nil,
      PChar(FAddress), nil, nil, SW_SHOWNORMAL)
    else ShellExecute(Application.Handle, nil,
      PChar('mailto:' + FAddress), nil, nil, SW_SHOWNORMAL);
end;
procedure TLink.Update;
begin
  if FMouseIn then begin
    if FClicked
      then Font := FClickedFont
      else Font := FOverFont;
  end else begin
    Font := FMainFont;
  end;
end;

end.
